@@ -23,7 +23,7 @@ test_that('Check aldvmm fitting function.', {
   utility[34, 1] <- NA
   utility[77, 2] <- NA
   
-  # With numerical gradients
+  # With Analytical gradients
   suppressMessages({
     suppressWarnings({testthat::expect_message(
       aldvmm(eq5d ~ age | female,
@@ -33,7 +33,7 @@ test_that('Check aldvmm fitting function.', {
     })
   })
   
-  # With gradient descent
+  # With finite difference
   suppressMessages({
     suppressWarnings({
       testthat::expect_message(
@@ -59,20 +59,20 @@ test_that('Check aldvmm fitting function.', {
     })
   })
   
-  # With numerical gradients
+  # With analytical gradients
   suppressMessages({
     suppressWarnings({
       fit <- aldvmm(eq5d ~ age | female,
                     data = utility,
                     psi = c(-0.594, 0.883),
-                    init.lo = rep(-99, length(fit$coef)))
+                    init.lo = rep(0.1, length(fit$coef)))
     })
   })
   
   testthat::expect(fit$optim.method == "L-BFGS-B",
                    failure_message = 'optim.method should be "L-BFGS-B"')  
   
-  # With gradient descent
+  # With finite difference
   suppressMessages({
     suppressWarnings({
       fit <- aldvmm(eq5d ~ age | female,
@@ -89,7 +89,7 @@ test_that('Check aldvmm fitting function.', {
   # Infeasible starting values
   #---------------------------
   
-  # With numerical gradients
+  # With analytical gradients
   suppressMessages({
     suppressWarnings({
       testthat::expect_error(aldvmm(eq5d ~ age | female,
@@ -100,7 +100,7 @@ test_that('Check aldvmm fitting function.', {
   })
   
   
-  # With gradient descent
+  # With finite difference
   suppressMessages({
     suppressWarnings({
       testthat::expect_error(aldvmm(eq5d ~ age | female,
